#include "grid.hpp"
#include <iostream>
#include <unistd.h>


Grid::Grid (Parameters& p) {

  _N[0] = p.getInt("Nx");
  _N[1] = p.getInt("Ny");

  // Check
  if (_N[0] < 3 or _N[1] <3) {
    std::cerr << "Grid : Insufficient or negative number of points : x " << _N[0] << " y " << _N[1] << std::endl;
    abort();
  }

  _h[0] = 1./(double)(_N[0]-1);
  _h[1] = 1./(double)(_N[1]-1);

}

double Grid::getSpacing (uint dim) {

  if (dim!=0 and dim!=1) {
    std::cerr << "Grid::getSpacing: wrong dimension " << dim << std::endl; 
    abort();
  }
  return _h[dim];

}

uint Grid::getNbPoints (uint dim) {

  if (dim!=0 and dim!=1) {
    std::cerr << "Grid::getNbPoints: wrong dimension " << dim << std::endl; 
    abort();
  }
  return _N[dim];

}

uint Grid::index1D(uint i, uint j) {

  if (i>=_N[0] or j>=_N[1]) {
    std::cerr << "Grid::index1D: given i,j " << i << "," << j << " but grid has size " << _N[0] << "," << _N[1] << std::endl; 
    abort();
  }

  return j*_N[0] + i;

}

void Grid::index2D(uint row, uint& i, uint& j) {

  if (row>=_N[0]*_N[1]) {
    std::cerr << "Grid::index2D: given row index (" << row << ")" << " is not in matrix of size " << _N[0]*_N[1] <<  std::endl; 
    abort();
  }
  j = row/_N[0];
  i = row%_N[0];

}

void Grid::coords(uint i, uint j, double& x, double& y) {
  x = i*_h[0];
  y = j*_h[1];
}

