#include "heatProblem.hpp"
#include <cmath>
#include <iostream>
#include <iomanip>

HeatProblem::HeatProblem(Parameters& p) {

  _params = &p;
  _k  = p.getReal("diffusion coefficient",1.);
  _t0 = p.getReal("start time",0.);
  _tf = p.getReal("end time",10.);
  _dt = p.getReal("time step",0.1);

  _Noutput = p.getInt("output frequency",2);
  
  _grid   = Grid(p);
  _solver = LinearSolver(&p,&_grid);
  _assbl  = HeatAssembler(&_grid,this);
  _writer = Writer(p,_grid);

}

void HeatProblem::run() {
  startUp();
  int iter=0;
  while (_t<_tf) {

    std::cout << "Time " << std::setw(8) << _t << "/" << _tf << " \t";
    _assbl.assembleRHS(_un,_t,_dt);
    _solver.solve(_assbl.getMatrix(),_assbl.getRHS(),_unP1);

    _un = _unP1;

    if (iter%_Noutput==0)
      _writer.write(_un);

    iter++;
    _t += _dt;
  }

}

void HeatProblem::startUp() {
  _t    = _t0;
  _un   = rvec(_grid.getNbPoints(0)*_grid.getNbPoints(1),0.);
  _unP1 = rvec(_grid.getNbPoints(0)*_grid.getNbPoints(1),0.);
  _assbl.assembleMatrix(_dt);
}

double HeatProblem::sourceTerm(double x,double y,double t) {

  if (t>2) return 0.;

  return exp(-(pow(x-0.5,2)+pow(y-0.5,2))/0.1);

}


