#ifndef _HEAT_PROBLEM_HPP_
#define _HEAT_PROBLEM_HPP_

#include "sparseMatrix.hpp"
#include "parameters.hpp"
#include "grid.hpp"
#include "assembler.hpp"
#include "linearSolver.hpp"
#include "writer.hpp"

class HeatProblem {

  public:

    HeatProblem(Parameters& p);

    void run();

    double sourceTerm(double x, double y, double t);

    double getDiffusionCoeff() {return _k;};

  private:

    void startUp();

    Parameters*   _params;
    Grid          _grid;
    LinearSolver  _solver;
    HeatAssembler _assbl;
    Writer        _writer;

    double _k; // diffusion coefficient
    double _t0,_tf; // Simulation times
    double _dt; // Time step
    double _t; // Current simulation time

    int _Noutput; // Output frequency

    rvec _un,_unP1; // solution at time t^n and t^{n+1}

};

#endif//_HEAT_PROBLEM_HPP_

