#include "writer.hpp"
#include <fstream>
#include <unistd.h>
#include <sstream>
#include <iomanip>

Writer::Writer(Parameters& p,Grid& g) {

  _params = &p;
  _grid   = &g;

  _baseName = p.getString("output file name");
  
  _nFiles = 0;

}

void Writer::write(const rvec& u) {

  writeHeader();

  std::ofstream ofile(getFileName(),std::ios_base::app);

  for (uint i=0;i<u.size();i++)
    ofile << u[i] << std::endl;

  _nFiles++;

}

std::string Writer::getFileName() {

  std::stringstream ss;
  ss << _baseName << std::setfill('0') << std::setw(3) << _nFiles << ".vtk";
  return ss.str();
}

void Writer::writeHeader() {
  std::ofstream of(getFileName().c_str());
  int M = _grid->getNbPoints(0);
  int N = _grid->getNbPoints(1);

  of << "# vtk DataFile Version 3.0" << std::endl;
  of << "Test heat equation" << std::endl;
  of << "ASCII" << std::endl;
  of << "DATASET STRUCTURED_POINTS" << std::endl;
  of << "DIMENSIONS " << M << " " << N << " 1" << std::endl;
  of << "ORIGIN 0. 0. 0." << std::endl;
  of << "SPACING " << _grid->getSpacing(0) << " " << _grid->getSpacing(1) << " 0." << std::endl;
  of << "POINT_DATA " << M*N << std::endl;
  of << "SCALARS u double" << std::endl;
  of << "LOOKUP_TABLE default" << std::endl;
}

